@@ -1,6 +1,6 @@
 /*  AO40TLMVIEW - decoder and viewer for AO-40 telemetry
  *
- *  Copyright (C) 2001, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
+ *  Copyright (C) 2001-2002, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of version 2 of the GNU General Public License as
@@ -22,6 +22,7 @@
 #include <ncurses.h>
 #include <math.h>
 #include <strings.h>
+#include <ctype.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -29,6 +30,8 @@
 #include <netdb.h>
 
 #include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/types.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <time.h>
@@ -38,7 +41,7 @@
 #include <sys/ioctl.h>
 #include <termios.h>
 
-#define VERSION "1.02"
+#define VERSION "1.03"
 
 typedef unsigned char byte;
 
@@ -1314,6 +1317,12 @@ int numblocks=0;     /* this is the number of blocks in the *ftlm file */
 FILE *flog=NULL;     /* the file to which telemetry blocks received via TCP or UDP are written; normally the same as ftlm */
 int curblock=-1;     /* indicates which block is being shown now.  0...numblocks-1 refer to blocks from ftlm; numblocks refers to partial block; -1 is no block (typically because ftlm is empty) */
 
+char atcallsign[16]="";   /* empty; or callsign of user, preceeded by '@'; included in names of log files (for the benefit of the telemetry archive) */
+int logfileday=0;         /* if nonzero: day of month included in log file name */
+int logfiledelay=3600;    /* maximum time (in seconds) between two consecutively received blocks that must still be written into the same logfile, even if we changed UTC day in the meantime */
+time_t timelastlogwrite=0;  /* time of last write to log file, for use with logfiledelay */
+
+
 
 void closetlm(void)
 {
@@ -1324,6 +1333,7 @@ void closetlm(void)
    partialblock=0; partiallen=0;
    numblocks=0;
    curblock=-1;
+   logfileday=0;
 }
 
 void opentlmfile(char *name)
@@ -1345,8 +1355,9 @@ void openlogfile(void)
    char name[32];
 
    time(&tt);
-   lt=localtime(&tt);
-   sprintf(name,"T%02i%02i%02i.RAW",lt->tm_year%100,lt->tm_mon+1,lt->tm_mday);
+   lt=gmtime(&tt);
+   sprintf(name,"T%02i%02i%02i%s.RAW",lt->tm_year%100,lt->tm_mon+1,lt->tm_mday,atcallsign);
+   logfileday = lt->tm_mday;
 
    flog=fopen(name,"a");
    if (!flog) return;
@@ -1355,6 +1366,24 @@ void openlogfile(void)
    curblock=numblocks-1;
 }
 
+void checklogfileday(void)
+{
+   /* Check whether we've moved to a new day; 
+      if so, reopen the log file, so its name contains the proper date for the archive.
+      However, check logfiledelay first
+   */
+   time_t tt;
+   struct tm *lt;
+
+   time(&tt);
+   if (tt < timelastlogwrite+logfiledelay) return;
+
+   lt=gmtime(&tt);
+   if (lt->tm_mday==logfileday) return;
+   closetlm();
+   openlogfile();
+}
+
 
 byte *loadblock(int number)
 {
@@ -1545,6 +1574,52 @@ void do_graph(int chan1,int chan2)
    if (d2) free(d2);
 }
 
+
+int do_graph_wod(byte *b)
+{
+   float d0[384];
+   float d1[384];
+   int samples,chan,orbit,ma,orbit0,ma0;
+   int n;
+   float x,x0;
+
+   if (  strncmp(b+2," Whole Orbit Data",17)!=0
+      || sscanf(b+27,"Samples: %i",&samples)!=1
+      || sscanf(b+41,"Captured Channel: #%4x",&chan)!=1
+      || sscanf(b+7*64+22,"#%2x%2x",&orbit,&ma)!=2
+      || sscanf(b+8*64-11,"#%2x%2x",&orbit0,&ma0)!=2
+      )
+      return 0;
+
+   /* Unfortunately, the high-order bits of the orbit number are not contained
+      in the WOD format, but it would be nice to have the correct orbit number on
+      the horizontal axis. So let's make a guess based on the orbit number in any
+      recent A or E blocks we may have seen.
+      (Actually, it would be better to only consider orbit numbers from recent
+      A blocks, not E blocks since those may be very old. But that would be
+      messier to implement, so let's first see whether the present solution is
+      practically acceptable.)
+   */
+   orbit += (orb_hi<<8);    
+   orbit0 += (orb_hi<<8);
+   if (orbit0 > 1+(orb_hi<<8)+orb_lo) { orbit0-=256; orbit-=256; }
+   if (orbit0 < orbit) orbit-=256;
+
+   x = orbit + ma/256.;
+   x0 = orbit0 + (ma0+0.5)/256.;
+   n = 0;
+   while (x<x0) {
+      d0[n]=x;
+      d1[n]= (tlmfns[chan-0x100])(b[n+64]);
+      x+=samples/256.;
+      n++;
+   }
+   if (n<=0) return 0;
+   if (!graph_gnuplot(n,d0,d1,NULL)) graph_ascii(n,d0,d1,NULL);
+   return 1;
+}
+
+
 /*--------------------------------------------------------------------------------------------------------------------------*/
 
 
@@ -1564,7 +1639,7 @@ void preparehelp(void)
       wattrset(w,A_BOLD);
       waddstr(w,"AO40TLMVIEW " VERSION "  -  a viewer for AMSAT-OSCAR 40 telemetry\n");
       wattrset(w,0);
-      waddstr(w,"                                          (c) 2001, pa3fwm@amsat.org");
+      waddstr(w,"                                     (c) 2001-2002, pa3fwm@amsat.org");
       wattrset(w,0);
 
       mvwaddstr(w,2,0,"Keystroke commands:\n");
@@ -1572,7 +1647,7 @@ void preparehelp(void)
       waddstr(w," a  - toggle decoding of A blocks         R - display channel number\n");
       waddstr(w," e  - toggle decoding of E blocks         x - hexadecimal display   \n");
       waddstr(w," c  - toggle decoding of blocks with incorrect CRC                  \n");
-      waddstr(w," g  - make graph                          \n");
+      waddstr(w," g  - make graph                          G - graph Whole Orbit Data\n");
       waddstr(w," f  - read telemetry blocks from file\n");
       waddstr(w," t u - receive telemetry through TCP resp. UDP (note: host and port\n"
                 "      as specified on commandline; TCP defaults to Goddard server)\n");
@@ -1663,6 +1738,7 @@ void mainloop(void)
    fd_set readfds;
    unsigned char decbuf[512];  /* a copy of the last block (A- or E-type) we decoded; we need this for the 'r' and 'R' commands in case the current block is not A- or E-type  */
    int decbufvalid=0;
+   struct timeval timeout;
 
    memset(decbuf,0,512);
 
@@ -1728,7 +1804,9 @@ void mainloop(void)
       i=1;
       if (tcpsocket>=i) i=tcpsocket+1;
       if (udpsocket>=i) i=udpsocket+1;
-      select( i, &readfds, NULL, NULL, NULL );
+      timeout.tv_sec = 60;
+      timeout.tv_usec = 0;
+      select( i, &readfds, NULL, NULL, &timeout );
       if (FD_ISSET(0,&readfds)) {
          c=wgetch(wprogstat);
          if (asciigraph_visible) {
@@ -1737,6 +1815,7 @@ void mainloop(void)
          }
          switch(c) {
             case 'g':
+               if (!ftlm || curblock<0) break;
                {
                   char *s;
                   int ch1,ch2;
@@ -1756,6 +1835,15 @@ void mainloop(void)
                   if (ch1>=0x100 && ch1<=0x1ff && (ch2==0 || (ch2>=0x100 && ch2<=0x1ff))) do_graph(ch1,ch2);
                }
                break;
+            case 'G':
+               if (curblock<0) break;
+               if (curblock==numblocks) break;
+               {
+                  byte *b;
+                  b=loadblock(curblock);
+                  if (b) do_graph_wod(b);
+               }
+               break;
             case 'f': 
                { 
                   char *newfname;
@@ -1820,11 +1908,12 @@ void mainloop(void)
                if (hexvisible) {
                   hexvisible=0;
                   do_prefresh();
-               } else if (ftlm) {
+               } else if (ftlm && curblock>=0) {
                   byte *b;
                   int l;
                   if (curblock<numblocks) { b=loadblock(curblock); l=512; }
                   else { b=partialblock; l=partiallen; }
+                  if (!b) break;
                   showhex(b,l);
                   hexvisible=1;
                }
@@ -1932,6 +2021,7 @@ void mainloop(void)
          if (curblock<0) curblock=0;
          if (numblocks==0 && !partialblock) { curblock=-1; redraw=0; }
       }
+      if (logfileday) checklogfileday();
       if (tcpsocket>=0 && FD_ISSET(tcpsocket,&readfds)) {
          byte *bp;
          byte *prevpartialblock=partialblock;
@@ -1939,7 +2029,7 @@ void mainloop(void)
          if (bp) {
             int crcok=checkcrc(bp);
             if (!crcok) bp[0]|=0x80;
-            if (flog) { fwrite(bp,512,1,flog); fflush(flog); }
+            if (flog) { fwrite(bp,512,1,flog); fflush(flog); time(&timelastlogwrite); }
             if (curblock==numblocks-1 && !prevpartialblock) curblock++;
             if (curblock==numblocks) redraw=1;
             numblocks++;
@@ -1954,7 +2044,7 @@ void mainloop(void)
          if (bp) {
             int crcok=checkcrc(bp);
             if (!crcok) bp[0]|=0x80;
-            if (flog) { fwrite(bp,512,1,flog); fflush(flog); }
+            if (flog) { fwrite(bp,512,1,flog); fflush(flog); time(&timelastlogwrite); }
             if (curblock==numblocks-1) curblock++;
             if (curblock==numblocks) redraw=1;
             numblocks++;
@@ -1978,10 +2068,51 @@ void usage(void)
    exit(1);
 }
 
+
+int sethostandport(int tcp,char *p)
+/* returns 0 on success */
+{
+   p=strtok(p,":");
+   if (!p) return 1;
+   if (strlen(p)>255) return 1;
+   if (tcp) strcpy(tcphostname,p); else strcpy(udphostname,p);
+   p=strtok(NULL,"");
+   if (!p) return 0;
+   if (tcp) tcpport=atoi(p); else udpport=atoi(p);
+   return 0;
+}
+
+void parse_rcfile(char *filename)
+{
+   char s[80],*p,*q;
+   FILE *f;
+   f=fopen(filename,"r");
+   if (!f) return;
+   while (fgets(s,80,f)) {
+      p=strtok(s," \t\n\r");
+      if (!p) continue;
+      q=p; while (*q) *q++=tolower(*q);
+      q=strtok(NULL," \t\n\r");
+      if (strncmp(p,"call",4)==0) {
+         if (!q) { puts("Missing callsign"); exit(1); }
+         strncpy(atcallsign+1,q,sizeof(atcallsign)-1);
+         atcallsign[0]='@';
+         atcallsign[sizeof(atcallsign)-1]=0;
+      } else if (strncmp(p,"tcp",3)==0) {
+         if (!q || sethostandport(1,q)) { puts("Error setting TCP host/port"); exit(1); }
+      } else if (strncmp(p,"udp",3)==0) {
+         if (!q || sethostandport(0,q)) { puts("Error setting UDP host/port"); exit(1); }
+      } else if (strncmp(p,"logfiledelay",3)==0) {
+         if (!q) { puts("Missing delay value"); exit(1); }
+         logfiledelay=atoi(q);
+      }
+   }
+   fclose(f);
+}
+
 void parse_commandline(int argc, char **argv)
 {
    int i;
-   char *p;
 
    do {
       i=getopt(argc,argv,"f:t:u:");
@@ -1991,24 +2122,12 @@ void parse_commandline(int argc, char **argv)
             opentlmfile(optarg);
             break;
          case 't':
-            p=strtok(optarg,":");
-            if (!p) usage();
-            if (strlen(p)>255) usage();
-            strcpy(tcphostname,p);
-            p=strtok(NULL,"");
-            if (!p) break;
-            tcpport=atoi(p);
+            if (sethostandport(1,optarg)) usage();
             tcpopen();
             if (tcpsocket>=0) openlogfile();
             break;
          case 'u':
-            p=strtok(optarg,":");
-            if (!p) usage();
-            if (strlen(p)>255) usage();
-            strcpy(udphostname,p);
-            p=strtok(NULL,"");
-            if (!p) break;
-            udpport=atoi(p);
+            if (sethostandport(0,optarg)) usage();
             udpopen();
             if (udpsocket>=0) openlogfile();
             break;
@@ -2039,6 +2158,18 @@ void sigconthandler(int i)
 
 int main(int argc,char **argv)
 {
+   char *s;
+
+   s=getenv("HOME");
+   if (s) {
+      int l;
+      l=strlen(s);
+      s=malloc(l+16);
+      strcpy(s,getenv("HOME"));
+      strcat(s,"/.ao40tlmviewrc");
+      parse_rcfile(s);
+   }
+   parse_rcfile(".ao40tlmviewrc");
    parse_commandline(argc,argv);
 
    signal(SIGWINCH, sigwinchhandler);
